"""
Tests for `pygpx` module.
"""
import datetime
import os

import xmltodict

import pytcx

with open(os.path.join(os.path.dirname(__file__), 'Watergrove.tcx')) as h:
    WATERGROVE = h.read()


def traverse(data, *args):
    for arg in args:
        data = data[arg]
    return data


def test_first_point():
    data = xmltodict.parse(WATERGROVE)
    trackpoint = traverse(
        data, 'TrainingCenterDatabase', 'Activities', 'Activity', 'Lap', 0, 'Track', 'Trackpoint',
        0
    )
    point = pytcx.Point(trackpoint)
    assert point.time == datetime.datetime(2017, 11, 25, 9, 2, 42, 1000)
    assert point.latitude == 53.657005447894335
    assert point.longitude == -2.131700534373522
    assert point.altitude == 241.60000610351562
    assert point.heart_rate == 108
    assert point.cadence == 0


def test_point_seven():
    data = xmltodict.parse(WATERGROVE)
    trackpoint = traverse(
        data, 'TrainingCenterDatabase', 'Activities', 'Activity', 'Lap', 0, 'Track', 'Trackpoint',
        7
    )
    point = pytcx.Point(trackpoint)
    assert point.time == datetime.datetime(2017, 11, 25, 9, 3, 10)
    assert point.latitude == 53.657675329595804
    assert point.longitude == -2.131626270711422
    assert point.altitude == 242.8000030517578
    assert point.heart_rate is None
    assert point.cadence == 80


def test_first_lap():
    data = xmltodict.parse(WATERGROVE)
    lap_data = traverse(
        data, 'TrainingCenterDatabase', 'Activities', 'Activity', 'Lap', 0,
    )
    lap = pytcx.Lap(lap_data)
    assert len(lap.points) == 62
    assert [x.time for x in lap.points[0:10]] == [
        datetime.datetime(2017, 11, 25, 9, 2, 42, 1000),
        datetime.datetime(2017, 11, 25, 9, 2, 43),
        datetime.datetime(2017, 11, 25, 9, 2, 49),
        datetime.datetime(2017, 11, 25, 9, 2, 51),
        datetime.datetime(2017, 11, 25, 9, 2, 54),
        datetime.datetime(2017, 11, 25, 9, 3, 1),
        datetime.datetime(2017, 11, 25, 9, 3, 3),
        datetime.datetime(2017, 11, 25, 9, 3, 10),
        datetime.datetime(2017, 11, 25, 9, 3, 13),
        datetime.datetime(2017, 11, 25, 9, 3, 14)
    ]
    assert lap.start() == datetime.datetime(2017, 11, 25, 9, 2, 42, 1000)
    assert lap.stop() == datetime.datetime(2017, 11, 25, 9, 9, 20)


def test_activity():
    data = xmltodict.parse(WATERGROVE)
    activity_data = traverse(
        data, 'TrainingCenterDatabase', 'Activities', 'Activity',
    )
    activity = pytcx.Activity(activity_data)
    assert len(activity.laps) == 6
    assert [x.start() for x in activity.laps] == [
        datetime.datetime(2017, 11, 25, 9, 2, 42, 1000),
        datetime.datetime(2017, 11, 25, 9, 9, 23),
        datetime.datetime(2017, 11, 25, 9, 16, 39),
        datetime.datetime(2017, 11, 25, 9, 23, 57),
        datetime.datetime(2017, 11, 25, 9, 30, 10),
        datetime.datetime(2017, 11, 25, 9, 36, 6),
    ]
    assert len(list(activity.points())) == 267
    assert activity.name == 'Wardle and West Littleborough Ward Running'
    assert activity.sport == 'Running'
    assert activity.start() == datetime.datetime(2017, 11, 25, 9, 2, 42, 1000)
    assert activity.stop() == datetime.datetime(2017, 11, 25, 9, 36, 11)


def test_parse_text():
    activities = pytcx.parse_to_activities(WATERGROVE)
    assert len(activities) == 1
    assert activities[0].name == 'Wardle and West Littleborough Ward Running'


"""A module for parsing tcx file into a list of activities."""
import datetime
import itertools
import xmltodict


__version__ = '0.1.0'


class Point(object):  # pylint: disable=too-few-public-methods
    """Represents a point in space-time.  Also includes TCX information such
    as heart rate and cadence."""
    def __init__(self, trackpoint):
        self.time = datetime.datetime.strptime(
            trackpoint['Time'], '%Y-%m-%dT%H:%M:%S.%fZ'
        )
        self.latitude = float(trackpoint['Position']['LatitudeDegrees'])
        self.longitude = float(trackpoint['Position']['LongitudeDegrees'])
        self.altitude = float(trackpoint['AltitudeMeters'])
        heart_rate = trackpoint.get('HeartRateBpm', {}).get('Value')
        if heart_rate is None:
            self.heart_rate = heart_rate
        else:
            self.heart_rate = float(heart_rate)
        self.cadence = float(trackpoint['Extensions']['TPX']['RunCadence'])


class Lap(object):
    """Represents a "lap".  Not necessarily round a course, but a section of a
    longer activity.  Frequently around 1 km or 1 mile depending on the user's
    settings."""
    def __init__(self, track):
        self.points = [Point(point) for point in track['Track']['Trackpoint']]

    def start(self):
        """Returns the first recorded time for the lap."""
        return self.points[0].time

    def stop(self):
        """Returns the last recorded time for the lap."""
        return self.points[-1].time


class Activity(object):
    """Represents a recorded activity.  An activity consistens of a number of
    laps, each with a number of points and in total records an entire
    workout."""
    def __init__(self, activity):
        self.laps = [Lap(lap) for lap in activity['Lap']]
        self.name = activity['Notes']
        self.sport = activity['@Sport']

    def start(self):
        """Returns the first recorded time for the activity."""
        return self.laps[0].start()

    def stop(self):
        """Returns the last recorded time for the activity."""
        return self.laps[-1].stop()

    def points(self):
        """Returns an iterator with all the points for the activity."""
        return itertools.chain(*[x.points for x in self.laps])


def parse_to_activities(text):
    """Parses the text from a TCX file into a list of activities."""
    data = xmltodict.parse(text)
    activity_data = data['TrainingCenterDatabase']['Activities']['Activity']
    if isinstance(activity_data, dict):
        activity_data = [activity_data]
    activities = [Activity(x) for x in activity_data]
    return activities
